unit uDirSync;
{$INCLUDE 'DirSync3.inc'}

interface

implementation

uses
  Windows, SysUtils, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  {$IFDEF DEBUG_LOGS} uLog, {$ENDIF}
  uSynchronizeDirectories,
  uSystem, uDirSyncConsts, uMessages, uDiscoverDifferences,
  uOptions, uOptionsDialog;

type
  EDirSyncPluginError = class(EFarPluginError);

  TDirSyncPlugin = class(TFarPlugin)
  private
    fPluginInfoAvailable: boolean;
    fPluginInfo: TPluginInfo;
  protected
    function PluginID: TGUID; override;
    function GetPluginTitle(out Description, Author: string): string; override;
    function GetPluginInfo(out Info: TPluginInfo): boolean; override;
    function Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle; override;
    function GetCommonString(ID: TFarPlugin.TCommonString): string; override;
    procedure DoLog(const Msg: string); override;
    function PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean; override;
    function ProcessMacroCall(const Params: array of Variant): boolean;
  public
    constructor Create;
    destructor Destroy; override;
  end;

const
  PLUGIN_TITLE = 'DirSync3';
  PLUGIN_DESCRIPTION = 'Synchronize directories';
  PLUGIN_AUTHOR = 'Pepak, http://www.pepak.net';

{ TDirSyncPlugin }

constructor TDirSyncPlugin.Create;
begin
  inherited;
  fPluginInfoAvailable := False;
end;

destructor TDirSyncPlugin.Destroy;
begin
  if fPluginInfoAvailable then begin
    DeletePluginInfo(fPluginInfo);
  end;
  inherited;
end;

procedure TDirSyncPlugin.DoLog(const Msg: string);
begin
  {$IFDEF DEBUG_LOGS}
  uLog.Log(Msg);
  {$ENDIF}
end;

function TDirSyncPlugin.Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle;
var
  Diff: TDiscoverDifferences;
  Editor: TSynchronizeTask;
begin
  Result := {$IFDEF FAR3_UP} 0 {$ELSE} INVALID_HANDLE_VALUE {$ENDIF} ;
  if OpenFrom = OPEN_EDITOR then
  begin
    if TSynchronizeTask.FindByEditorID(-1, Editor) then
    begin
      Editor.ExecuteTask;
    end;
  end
  {$IFDEF FAR3_UP}
  else if OpenFrom = OPEN_FROMMACRO_ then
  begin
    if ProcessMacroCall(TFarMacro.MacroParamsToDelphi(Data)) then
      Result := 1;
  end
  {$ENDIF}
  else begin
    Diff := TDiscoverDifferences.Create;
    try
      Result := Diff.Execute;
    finally
      FreeAndNil(Diff);
    end;
  end;
end;

function TDirSyncPlugin.GetCommonString(ID: TFarPlugin.TCommonString): string;
begin
  case ID of
    TFarPlugin.TCommonString.MTitle:
      Result := GetMsg(MTitle);
    TFarPlugin.TCommonString.MExceptionOccurredIn:
      Result := GetMsg(MExceptionOccurredIn);
    TFarPlugin.TCommonString.MOk:
      Result := GetMsg(MOk);
    TFarPlugin.TCommonString.MCancel:
      Result := GetMsg(MCancel);
    TFarPlugin.TCommonString.MYes:
      Result := GetMsg(MYes);
    TFarPlugin.TCommonString.MNo:
      Result := GetMsg(MNo);
    else
      Result := inherited GetCommonString(ID);
  end;
end;

function TDirSyncPlugin.GetPluginInfo(out Info: TPluginInfo): boolean;
var
  fMainMenu: TFarPlugin.TFarPluginMenu;
  fConfigMenu: TFarPlugin.TFarPluginMenu;
  fDiskMenu: TFarPlugin.TFarPluginMenu;
begin
  if not fPluginInfoAvailable then begin
    SetLength(fMainMenu, 1);
    fMainMenu[0].Title := GetMsg(MTitle);
    fMainMenu[0].ID := MENU_PANEL_GUID;
    SetLength(fConfigMenu, 1);
    fConfigMenu[0].Title := GetMsg(MTitle);
    fConfigMenu[0].ID := MENU_CONFIG_GUID;
    SetLength(fDiskMenu, 0);
    Self.fPluginInfo := CreatePluginInfo(
      PF_EDITOR,
      fMainMenu, fConfigMenu, fDiskMenu,
      ''
    );
    fPluginInfoAvailable := True;
  end;
  Info := Self.fPluginInfo;
  Result := True;
end;

function TDirSyncPlugin.GetPluginTitle(out Description, Author: string): string;
begin
  Result := PLUGIN_TITLE;
  Description := PLUGIN_DESCRIPTION;
  Author := PLUGIN_AUTHOR;
end;

function TDirSyncPlugin.PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean;
var
  Dlg: TOptionsDialog;
begin
  Result := False;
  Dlg := TOptionsDialog.Create;
  try
    Dlg.Options := TDirSyncOptions.Create;
    try
      Dlg.Options.Load;
      if Dlg.Execute then
        Result := True;
    finally
      Dlg.Options.Free;
      Dlg.Options := nil;
    end;
  finally
    FreeAndNil(Dlg);
  end;
end;

function TDirSyncPlugin.PluginID: TGUID;
begin
  Result := PLUGIN_GUID;
end;

function TDirSyncPlugin.ProcessMacroCall(const Params: array of Variant): boolean;

  function GetDirectionFromParameter(ParamIndex: integer): boolean;
    // returns True for Left, False for Right, Exception otherwise
  var
    Value: string;
  begin
    if Length(Params) <= ParamIndex then
      raise EDirSyncPluginError.CreateFmt(GetMsg(MMissingMacroParam), [ParamIndex+1]);
    Value := Params[ParamIndex];
    if Value = 'left' then
      Result := True
    else if Value = 'right' then
      Result := False
    else
      raise EDirSyncPluginError.CreateFmt(GetMsg(MInvalidMacroDirection), [Value]);
  end;

var
  FunctionName: string;
  Editor: TSynchronizeTask;
begin
  Result := False;
  if not TSynchronizeTask.FindByEditorID(-1, Editor) then
    Editor := nil;
  if Length(Params) > 0 then
  begin
    FunctionName := Params[0];
    if FunctionName = 'is_editor' then
    begin
      if Editor <> nil then
        Result := True;
    end
    else if Editor = nil then
    begin
      Result := False;
    end
    else if FunctionName = 'change_direction' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etCopyToLeft)
      else
        Result := Editor.ExecuteTask(etCopyToRight);
    end
    else if FunctionName = 'next_difference' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etFindNextToLeft)
      else
        Result := Editor.ExecuteTask(etFindNextToRight);
    end
    else if FunctionName = 'edit_file' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etEditLeft)
      else
        Result := Editor.ExecuteTask(etEditRight);
    end
    else if FunctionName = 'goto_file' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etGotoLeft)
      else
        Result := Editor.ExecuteTask(etGotoRight);
    end
    else if FunctionName = 'file_info' then
    begin
      Result := Editor.ExecuteTask(etFileInfo);
    end
    else if FunctionName = 'compare' then
    begin
      Result := Editor.ExecuteTask(etCompare);
    end;
  end;
end;

exports
  ProcessEditorEventW,
  ProcessEditorInputW;

initialization
  TFarPlugin.InitInstance(TDirSyncPlugin.Create);

end.
